# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Miscellaneous stuff that might move into bzrlib or qbzr one day.

In some cases, the code is already in bzrlib but in a later version
than the minimum version required.
"""

from PyQt4 import QtCore, QtGui

from bzrlib.config import ConfigObj
from bzrlib.osutils import joinpath, splitpath


def reload_config_obj(cfg, filename):
    """Reload a Bazaar Config object from a file."""
    try:
        stream = open(filename, 'r')
    except IOError:
        lines = []
    else:
        lines = stream.readlines()
        stream.close()
    cfg._parser = ConfigObj(lines, encoding='utf-8')


def parent_directories(filename):
    """Return the list of parent directories, deepest first.
    
    For example, parent_directories("a/b/c") -> ["a/b", "a"].
    """
    parents = []
    parts = splitpath(dirname(filename))
    while parts:
        parents.append(joinpath(parts))
        parts.pop()
    return parents


def build_combo_with_labels(data_label_tuples):
    """Build a combo-box with labelled data value."""
    combo = QtGui.QComboBox()
    for data, label in data_label_tuples:
        combo.addItem(label, QtCore.QVariant(data))
    return combo
