# Copyright (C) 2009 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from PyQt4 import QtCore, QtGui

from bzrlib.plugins.explorer.lib import (
    kinds,
    tools,
    )
from bzrlib.plugins.explorer.lib.i18n import gettext, N_


class Toolbox(QtGui.QDockWidget):
    """Panel for selecting tools.

    Tools are organised into toolsets. Each toolset is a 'tab'
    within the toolbox. Within each toolset, tools are provided
    as tool buttons.
    """

    def __init__(self, action_callback, profiles_provider, tool_runner, *args):
        self._action_callback = action_callback
        self._profiles_provider = profiles_provider
        self._tool_runner = tool_runner
        QtGui.QDockWidget.__init__(self, *args)
        self.setObjectName("ToolboxDockWidget")
        self.refresh()

    def refresh(self):
        self.setWidget(self._build_ui())

    def _build_ui(self):
        tbox = QtGui.QToolBox()
        profiles = self._profiles_provider()
        for prof in reversed(profiles):
            added_actions = 0
            tbar = self._empty_toolbar()
            root_entry = prof.tools().root()
            for entry in root_entry:
                added_actions += self._add_tool_entry_to_toolbar(entry, tbar, tbox)
            if added_actions:
                tbox.addItem(tbar, root_entry.title)
        return tbox

    def _add_tool_entry_to_toolbar(self, entry, tbar, tbox):
        """Return the number of actions added to tbar."""
        if isinstance(entry, tools.Tool):
            def open_callback():
                self._tool_runner(entry)
            label = entry.title
            action = tbar.addAction(entry.title, open_callback)
            tip = "(%s) %s" % (entry.type, entry.action)
            action.setStatusTip(tip)
            if entry.icon:
                icon = kinds.icon_by_resource_path(entry.icon)
            else:
                icon = kinds.icon_for_kind(entry.type)
            action.setIcon(icon)
            return 1
        elif isinstance(entry, tools.ToolFolder):
            sub_tbar = self._empty_toolbar()
            for child in entry:
                self._add_tool_entry_to_toolbar(child, sub_tbar, tbox)
            tbox.addItem(sub_tbar, entry.title)
            return 0
        elif isinstance(entry, tools.ToolSeparator):
            # Ignore separators in toolboxes
            return 0

    def _empty_toolbar(self):
        tb = QtGui.QToolBar()
        tb.setOrientation(QtCore.Qt.Vertical)
        tb.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
        return tb
